/* 
 *  Lsystem version 1.1, Copyright (C) 1994 G. Hutchings
 *  Lsystem comes with ABSOLUTELY NO WARRANTY.
 *  This is free software, and you are welcome to redistribute it
 *  under certain conditions; see the file COPYING for details.
 */

/* Drawing functions */

#include <stdio.h>
#include <math.h>
#include "draw.h"
#include "defs.h"

#define UNDEF	-999.0		 /* an undefined value */

double x = UNDEF, y = UNDEF;	 /* current position */
double heading = 0.0;		 /* current heading (degrees) */

double red = UNDEF;
double green = UNDEF;		 /* current colour */
double blue = UNDEF;

extern double angle_inc;	 /* angle increment */
extern FILE *ofp;		 /* output stream */

struct state *stack;		 /* turtle state stack */
int stack_ptr = 0;		 /* current stack size */
int stack_max = MAX_STATES;	 /* initial max. stack size */

/* Turn string into drawing commands and write them */
void
draw(char *str)
{
    char *macro;

    /* Allocate turtle stack */
    stack = ALLOC(struct state, stack_max);
    if (stack == NULL) MEMERROR;

    /* Set colour and move to initial position */
    setcolour(0.0, 0.0, 0.0);
    moveto(0.0, 0.0);

    /* Parse string */
    while (*str != NULL) {
	if ((macro = expandmacro(*str)) == NULL) {
	    dochar(*str);
	} else {
	    while (*macro != NULL) dochar(*macro++);
	}

	str++;
    }

    /* Deallocate stack */
    DEALLOC(stack);
}

/* Output commands based on character */
void
dochar(char c)
{
    double r, g, b;

    /* Colour commands */
    if (findcolour(c, &r, &g, &b)) setcolour(r, g, b);

    /* Movement/drawing commands */
    switch (c) {
    case FORWARD_NODRAW:
	moveto(x+sin(heading), y+cos(heading));
	break;
    case FORWARD_DRAW:
	lineto(x+sin(heading), y+cos(heading));
	break;
    case TURN_CLOCKWISE:
	heading += angle_inc;
	break;
    case TURN_ANTICLOCKWISE:
	heading -= angle_inc;
	break;
    case PUSH_STATE:
	/* Push variables onto stack */
	stack[stack_ptr].x = x;
	stack[stack_ptr].y = y;
	stack[stack_ptr].heading = heading;
	stack[stack_ptr].red = red;
	stack[stack_ptr].green = green;
	stack[stack_ptr].blue = blue;
	stack_ptr++;

	/* Expand stack if required */
	if (stack_ptr == stack_max) {
	    stack_max *= 2;
	    stack = REALLOC(stack, struct state, stack_max);
	    if (stack == NULL) MEMERROR;
	}
	break;
    case POP_STATE:
	if (stack_ptr > 0) {
	    stack_ptr--;
	    heading = stack[stack_ptr].heading;
	    moveto(stack[stack_ptr].x, stack[stack_ptr].y);
	    setcolour(stack[stack_ptr].red,
		      stack[stack_ptr].green,
		      stack[stack_ptr].blue);
	}
	break;
    }
}

/* Write colour command */
void
setcolour(double r, double g, double b)
{
    double diff = ABS(red-r)+ABS(green-g)+ABS(blue-b);

    if (diff > 0.0) {
	red = r;
	green = g;
	blue = b;
	fprintf(ofp, "C %.3f %.3f %.3f\n", red, green, blue);
    }
}

/* Write move command */
void
moveto(double xp, double yp)
{
    double diff = ABS(xp-x)+ABS(yp-y);

    if (diff > 0.0) {
	x = xp;
	y = yp;
	fprintf(ofp, "M %.3f %.3f\n", x, y);
    }
}

/* Write line command */
void
lineto(double xp, double yp)
{
    double diff = ABS(xp-x)+ABS(yp-y);

    if (diff > 0.0) {
	x = xp;
	y = yp;
	fprintf(ofp, "L %.3f %.3f\n", x, y);
    }
}
