/* 
 *  Lsystem version 1.1, Copyright (C) 1994 G. Hutchings
 *  Lsystem comes with ABSOLUTELY NO WARRANTY.
 *  This is free software, and you are welcome to redistribute it
 *  under certain conditions; see the file COPYING for details.
 */

/* Driver routine */

#include <stdio.h>
#include "defs.h"
#include "date.h"
#include "version.h"

char input_file[MAX_STR_LEN];	 /* input filename */
char output_file[MAX_STR_LEN];	 /* output filename */

int line_num = 0;		 /* input line number */
int num_errors = 0;		 /* no. of compilation errors */
int stringinfo = 0;		 /* whether to print string info */

#ifdef DEBUG
extern int yydebug;		 /* parser debugging flag */
#endif

extern FILE *yyin;		 /* parser input stream */

FILE *ofp = stdout;		 /* output stream */

int
main(int argc, char *argv[])
{
    int c;
    char *str;
    extern char *optarg;
    extern int optind, opterr, initialized;

    /* Parse arguments */
    opterr = 0;
    strcpy(input_file, "<stdin>");
    strcpy(output_file, "<stdout>");

    while ((c = getopt(argc, argv, "o:svD")) != EOF) {
	switch (c) {
	case 'o':
	    strcpy(output_file, optarg);
	    ofp = fopen(output_file, "w");
	    if (ofp == NULL) {
		fprintf(stderr, "Can't open %s\n", output_file);
		exit(1);
	    }
	    break;
	case 's':
	    stringinfo = 1;
	    break;
	case 'v':
	    printf("%s\n", version);
	    printf("Compiled on %s by %s@%s\n", DATE, USER, HOST);
	    exit(0);
#ifdef DEBUG
	case 'D':		 /* parser debug option */
	    yydebug = 1;
	    break;
#endif
	default:
	    usage();
	}
    }

    /* Last argument (if any) is input file */
    if (optind != argc) {
	strcpy(input_file, argv[optind]);
	yyin = fopen(input_file, "r");
	if (yyin == NULL) {
	    fprintf(stderr, "Can't open %s\n", input_file);
	    exit(1);
	}
    }

    /* Compile input */
    yyparse();
    if (yyin != stdin) fclose(yyin);
    if (!initialized) fatal("no init command found");

    /* Abort if errors found */
    if (num_errors > 0) return num_errors;

    /* Iterate initial string according to rules */
    str = iterate();

    /* Draw it */
    draw(str);

    /* Close output */
    if (ofp != stdout) fclose(ofp);

    /* Er... that's it */
    return 0;
}

/* Give a compilation error */
void
yyerror(char *msg)
{
    fprintf(stderr, "lsystem: Error: %s, line %d: %s\n",
	    input_file, line_num, msg);
    num_errors++;
}

/* Give a fatal error */
void
fatal(char *msg)
{
    fprintf(stderr, "lsystem: Fatal: %s\n", msg);
    exit(1);
}

/* Usage message */
void
usage()
{
    fprintf(stderr, "Usage: lsystem [options] [file]\n\n");
    fprintf(stderr, "-o file\t\twrite output to file\n");
    fprintf(stderr, "-s\t\tprint string length info\n");
    fprintf(stderr, "-v\t\tprint version info\n");
    exit(1);
}
